%----------------------------------
% Demo: Robust beamforming via SOCP

% by  Wing-Kin Ma and Jiaxian Pan, March 2013
%

% Problem 1:Minimum variance beamforming
%
% min  w^H R w 
% s.t. w^H a=1


%----------
% Problem 2: Robust beamforming 
%
% min  w^H R w
% s.t. |w^H (a + u)| >= 1, for all ||u||_2 <= epsi


%----------
m= 10; %array size
epsi= 0.2*sqrt(m); %the robustness parameter

angle_des= 20; %the look direction we thought
angle_act= 20.1;   %the actual look direction

R= zeros(m,m);
at= steering_vec(angle_act*(pi/180),m);
at2= steering_vec(40*(pi/180),m); %an interferer 
at3= steering_vec(-50*(pi/180),m); %another interferer
%synthesize the covariance matrix (don't do this in your sim)
R= at*at'+at2*at2'+at3*at3';
R= R + 0.01*eye(m); %add noise covariance
R= (R+R')/2 %eliminate numerical errors

a_des= steering_vec(angle_des*(pi/180),m);
inv_R= inv(R);
w_mv= inv_R*a_des/real(a_des'*inv_R*a_des); %MV beamformer
w_rmv_cvx= rmv_socp_cvx(R,a_des,epsi); %robust MV beamformer by CVX

theta= -pi/2:(pi/1000):pi/2;
P1= zeros(length(theta),1); P2= zeros(length(theta),1);
for i=1:length(theta)
    P_mv(i)= abs(w_mv'*steering_vec(theta(i),m)).^2;
    P_rmv_cvx(i)= abs(w_rmv_cvx'*steering_vec(theta(i),m)).^2;
end;

figure(1); clf;
plot(theta*(2/pi*90),10*log10(P_mv),'linewidth',2); hold on;
plot(theta*(2/pi*90),10*log10(P_rmv_cvx),'r--','linewidth',2);
xlabel('Angle (Degree)'); ylabel('Magnitude (dB)');
legend('MV',['Robust MV, epsilon= ',num2str(epsi)]);
set(gca,'XLim',[-90,90]);